@@ -1,7 +1,5 @@
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.79 $
-# Date: $Date: 2004/10/19 23:29:44 $
+# $Id: frontend.py 5502 2008-01-14 16:43:34Z goodger $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -32,13 +30,18 @@ import os
 import os.path
 import sys
 import types
-import copy
 import warnings
 import ConfigParser as CP
 import codecs
 import docutils
-import optparse
-from optparse import SUPPRESS_HELP
+import docutils.utils
+import docutils.nodes
+try:
+    import optparse
+    from optparse import SUPPRESS_HELP
+except ImportError:
+    import optik as optparse
+    from optik import SUPPRESS_HELP
 
 
 def store_multiple(option, opt, value, parser, *args, **kwargs):
@@ -116,7 +119,7 @@ def validate_encoding_and_error_handler(
 
 def validate_boolean(setting, value, option_parser,
                      config_parser=None, config_section=None):
-    if isinstance(value, types.StringType):
+    if isinstance(value, types.UnicodeType):
         try:
             return option_parser.booleans[value.strip().lower()]
         except KeyError:
@@ -124,6 +127,13 @@ def validate_boolean(setting, value, option_parser,
                    None, sys.exc_info()[2])
     return value
 
+def validate_nonnegative_int(setting, value, option_parser,
+                             config_parser=None, config_section=None):
+    value = int(value)
+    if value < 0:
+        raise ValueError('negative value; must be positive or zero')
+    return value
+
 def validate_threshold(setting, value, option_parser,
                        config_parser=None, config_section=None):
     try:
@@ -137,7 +147,7 @@ def validate_threshold(setting, value, option_parser,
 
 def validate_colon_separated_string_list(
     setting, value, option_parser, config_parser=None, config_section=None):
-    if isinstance(value, types.StringType):
+    if isinstance(value, types.UnicodeType):
         value = value.split(':')
     else:
         last = value.pop()
@@ -153,13 +163,26 @@ def validate_url_trailing_slash(
     else:
         return value + '/'
 
-def validate_dependency_file(
-    setting, value, option_parser, config_parser=None, config_section=None):
+def validate_dependency_file(setting, value, option_parser,
+                             config_parser=None, config_section=None):
     try:
         return docutils.utils.DependencyList(value)
     except IOError:
         return docutils.utils.DependencyList(None)
 
+def validate_strip_class(setting, value, option_parser,
+                         config_parser=None, config_section=None):
+    if config_parser:                   # validate all values
+        class_values = value
+    else:                               # just validate the latest value
+        class_values = [value[-1]]
+    for class_value in class_values:
+        normalized = docutils.nodes.make_id(class_value)
+        if class_value != normalized:
+            raise ValueError('invalid class value %r (perhaps %r?)'
+                             % (class_value, normalized))
+    return value
+
 def make_paths_absolute(pathdict, keys, base_path=None):
     """
     Interpret filesystem path settings relative to the `base_path` given.
@@ -209,6 +232,10 @@ class Values(optparse.Values):
                     del other_dict[setting]
         self._update_loose(other_dict)
 
+    def copy(self):
+        """Return a shallow copy of `self`."""
+        return self.__class__(defaults=self.__dict__)
+
 
 class Option(optparse.Option):
 
@@ -275,8 +302,9 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
     settings_spec = (
         'General Docutils Options',
         None,
-        (('Include a "Generated by Docutils" credit and link at the end '
-          'of the document.',
+        (('Specify the document title as metadata.',
+          ['--title'], {}),
+         ('Include a "Generated by Docutils" credit and link.',
           ['--generator', '-g'], {'action': 'store_true',
                                   'validator': validate_boolean}),
          ('Do not include a generator credit.',
@@ -284,141 +312,135 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
          ('Include the date at the end of the document (UTC).',
           ['--date', '-d'], {'action': 'store_const', 'const': '%Y-%m-%d',
                              'dest': 'datestamp'}),
-         ('Include the time & date at the end of the document (UTC).',
+         ('Include the time & date (UTC).',
           ['--time', '-t'], {'action': 'store_const',
                              'const': '%Y-%m-%d %H:%M UTC',
                              'dest': 'datestamp'}),
          ('Do not include a datestamp of any kind.',
           ['--no-datestamp'], {'action': 'store_const', 'const': None,
                                'dest': 'datestamp'}),
-         ('Include a "View document source" link (relative to destination).',
+         ('Include a "View document source" link.',
           ['--source-link', '-s'], {'action': 'store_true',
                                     'validator': validate_boolean}),
-         ('Use the supplied <URL> verbatim for a "View document source" '
-          'link; implies --source-link.',
+         ('Use <URL> for a source link; implies --source-link.',
           ['--source-url'], {'metavar': '<URL>'}),
          ('Do not include a "View document source" link.',
           ['--no-source-link'],
           {'action': 'callback', 'callback': store_multiple,
            'callback_args': ('source_link', 'source_url')}),
-         ('Enable backlinks from section headers to table of contents '
-          'entries.  This is the default.',
+         ('Link from section headers to TOC entries.  (default)',
           ['--toc-entry-backlinks'],
           {'dest': 'toc_backlinks', 'action': 'store_const', 'const': 'entry',
            'default': 'entry'}),
-         ('Enable backlinks from section headers to the top of the table of '
-          'contents.',
+         ('Link from section headers to the top of the TOC.',
           ['--toc-top-backlinks'],
           {'dest': 'toc_backlinks', 'action': 'store_const', 'const': 'top'}),
          ('Disable backlinks to the table of contents.',
           ['--no-toc-backlinks'],
           {'dest': 'toc_backlinks', 'action': 'store_false'}),
-         ('Enable backlinks from footnotes and citations to their '
-          'references.  This is the default.',
+         ('Link from footnotes/citations to references. (default)',
           ['--footnote-backlinks'],
           {'action': 'store_true', 'default': 1,
            'validator': validate_boolean}),
          ('Disable backlinks from footnotes and citations.',
           ['--no-footnote-backlinks'],
           {'dest': 'footnote_backlinks', 'action': 'store_false'}),
-         ('Disable Docutils section numbering',
-          ['--no-section-numbering'],
-          {'action': 'store_false', 'dest': 'sectnum_xform',
+         ('Enable section numbering.  (default)',
+          ['--section-numbering'],
+          {'action': 'store_true', 'dest': 'sectnum_xform',
            'default': 1, 'validator': validate_boolean}),
-         ('Set verbosity threshold; report system messages at or higher than '
-          '<level> (by name or number: "info" or "1", warning/2, error/3, '
-          'severe/4; also, "none" or "5").  Default is 2 (warning).',
+         ('Disable section numbering.',
+          ['--no-section-numbering'],
+          {'action': 'store_false', 'dest': 'sectnum_xform'}),
+         ('Remove comment elements from the document tree.',
+          ['--strip-comments'],
+          {'action': 'store_true', 'validator': validate_boolean}),
+         ('Leave comment elements in the document tree. (default)',
+          ['--leave-comments'],
+          {'action': 'store_false', 'dest': 'strip_comments'}),
+         ('Remove all elements with classes="<class>" from the document tree. '
+          'Warning: potentially dangerous; use with caution. '
+          '(Multiple-use option.)',
+          ['--strip-elements-with-class'],
+          {'action': 'append', 'dest': 'strip_elements_with_classes',
+           'metavar': '<class>', 'validator': validate_strip_class}),
+         ('Remove all classes="<class>" attributes from elements in the '
+          'document tree. Warning: potentially dangerous; use with caution. '
+          '(Multiple-use option.)',
+          ['--strip-class'],
+          {'action': 'append', 'dest': 'strip_classes',
+           'metavar': '<class>', 'validator': validate_strip_class}),
+         ('Report system messages at or higher than <level>: "info" or "1", '
+          '"warning"/"2" (default), "error"/"3", "severe"/"4", "none"/"5"',
           ['--report', '-r'], {'choices': threshold_choices, 'default': 2,
                                'dest': 'report_level', 'metavar': '<level>',
                                'validator': validate_threshold}),
-         ('Report all system messages, info-level and higher.  (Same as '
-          '"--report=info".)',
-          ['--verbose', '-v'], {'action': 'store_const', 'const': 'info',
+         ('Report all system messages.  (Same as "--report=1".)',
+          ['--verbose', '-v'], {'action': 'store_const', 'const': 1,
                                 'dest': 'report_level'}),
-         ('Do not report any system messages.  (Same as "--report=none".)',
-          ['--quiet', '-q'], {'action': 'store_const', 'const': 'none',
+         ('Report no system messages.  (Same as "--report=5".)',
+          ['--quiet', '-q'], {'action': 'store_const', 'const': 5,
                               'dest': 'report_level'}),
-         ('Set the threshold (<level>) at or above which system messages are '
-          'converted to exceptions, halting execution immediately by '
-          'exiting (or propagating the exception if --traceback set).  '
-          'Levels as in --report.  Default is 4 (severe).',
+         ('Halt execution at system messages at or above <level>.  '
+          'Levels as in --report.  Default: 4 (severe).',
           ['--halt'], {'choices': threshold_choices, 'dest': 'halt_level',
                        'default': 4, 'metavar': '<level>',
                        'validator': validate_threshold}),
-         ('Same as "--halt=info": halt processing at the slightest problem.',
+         ('Halt at the slightest problem.  Same as "--halt=info".',
           ['--strict'], {'action': 'store_const', 'const': 'info',
                          'dest': 'halt_level'}),
-         ('Enable a non-zero exit status for normal exit if non-halting '
-          'system messages (at or above <level>) were generated.  Levels as '
-          'in --report.  Default is 5 (disabled).  Exit status is the maximum '
-          'system message level plus 10 (11 for INFO, etc.).',
+         ('Enable a non-zero exit status for non-halting system messages at '
+          'or above <level>.  Default: 5 (disabled).',
           ['--exit-status'], {'choices': threshold_choices,
                               'dest': 'exit_status_level',
                               'default': 5, 'metavar': '<level>',
                               'validator': validate_threshold}),
-         ('Report debug-level system messages and generate diagnostic output.',
+         ('Enable debug-level system messages and diagnostics.',
           ['--debug'], {'action': 'store_true', 'validator': validate_boolean}),
-         ('Do not report debug-level system messages or generate diagnostic '
-          'output.',
+         ('Disable debug output.  (default)',
           ['--no-debug'], {'action': 'store_false', 'dest': 'debug'}),
-         ('Send the output of system messages (warnings) to <file>.',
+         ('Send the output of system messages to <file>.',
           ['--warnings'], {'dest': 'warning_stream', 'metavar': '<file>'}),
-         ('Enable Python tracebacks when halt-level system messages and '
-          'other exceptions occur.  Useful for debugging, and essential for '
-          'issue reports.',
+         ('Enable Python tracebacks when Docutils is halted.',
           ['--traceback'], {'action': 'store_true', 'default': None,
                             'validator': validate_boolean}),
-         ('Disable Python tracebacks when errors occur; report just the error '
-          'instead.  This is the default.',
+         ('Disable Python tracebacks.  (default)',
           ['--no-traceback'], {'dest': 'traceback', 'action': 'store_false'}),
-         ('Specify the encoding of input text.  Default is locale-dependent.  '
-          'Optionally also specify the error handler for undecodable '
-          'characters, after a colon (":"); default is "strict".  (See '
-          '"--intput-encoding-error-handler".)',
+         ('Specify the encoding and optionally the '
+          'error handler of input text.  Default: <locale-dependent>:strict.',
           ['--input-encoding', '-i'],
           {'metavar': '<name[:handler]>',
            'validator': validate_encoding_and_error_handler}),
-         ('Specify the error handler for undecodable characters in '
-          'the input.  Acceptable values include "strict", "ignore", and '
-          '"replace".  Default is "strict".  '
-          'Usually specified as part of --input-encoding.',
+         ('Specify the error handler for undecodable characters.  '
+          'Choices: "strict" (default), "ignore", and "replace".',
           ['--input-encoding-error-handler'],
           {'default': 'strict', 'validator': validate_encoding_error_handler}),
-         ('Specify the text encoding for output.  Default is UTF-8.  '
-          'Optionally also specify the error handler for unencodable '
-          'characters, after a colon (":"); default is "strict".  (See '
-          '"--output-encoding-error-handler".)',
+         ('Specify the text encoding and optionally the error handler for '
+          'output.  Default: UTF-8:strict.',
           ['--output-encoding', '-o'],
           {'metavar': '<name[:handler]>', 'default': 'utf-8',
            'validator': validate_encoding_and_error_handler}),
-         ('Specify the error handler for unencodable characters in '
-          'the output.  Acceptable values include "strict", "ignore", '
-          '"replace", "xmlcharrefreplace", and '
-          '"backslashreplace" (in Python 2.3+).  Default is "strict".  '
-          'Usually specified as part of --output-encoding.',
+         ('Specify error handler for unencodable output characters; '
+          '"strict" (default), "ignore", "replace", '
+          '"xmlcharrefreplace", "backslashreplace" (Python 2.3+).',
           ['--output-encoding-error-handler'],
           {'default': 'strict', 'validator': validate_encoding_error_handler}),
-         ('Specify the text encoding for error output.  Default is ASCII.  '
-          'Optionally also specify the error handler for unencodable '
-          'characters, after a colon (":"); default is "%s".  (See '
-          '"--output-encoding-error-handler".)'
+         ('Specify text encoding and error handler for error output.  '
+          'Default: ASCII:%s.'
           % default_error_encoding_error_handler,
           ['--error-encoding', '-e'],
           {'metavar': '<name[:handler]>', 'default': 'ascii',
            'validator': validate_encoding_and_error_handler}),
          ('Specify the error handler for unencodable characters in '
-          'error output.  See --output-encoding-error-handler for acceptable '
-          'values.  Default is "%s".  Usually specified as part of '
-          '--error-encoding.' % default_error_encoding_error_handler,
+          'error output.  Default: %s.'
+          % default_error_encoding_error_handler,
           ['--error-encoding-error-handler'],
           {'default': default_error_encoding_error_handler,
            'validator': validate_encoding_error_handler}),
-         ('Specify the language of input text (ISO 639 2-letter identifier).'
-          '  Default is "en" (English).',
+         ('Specify the language (as 2-letter code).  Default: en.',
           ['--language', '-l'], {'dest': 'language_code', 'default': 'en',
                                  'metavar': '<name>'}),
-         ('Write dependencies (caused e.g. by file inclusions) to '
-          '<file>.  Useful in conjunction with programs like "make".',
+         ('Write output file dependencies to <file>.',
           ['--record-dependencies'],
           {'metavar': '<file>', 'validator': validate_dependency_file,
            'default': None}),           # default set in Values class
@@ -429,6 +451,9 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           ['--version', '-V'], {'action': 'version'}),
          ('Show this help message and exit.',
           ['--help', '-h'], {'action': 'help'}),
+         # Typically not useful for non-programmatical use:
+         (SUPPRESS_HELP, ['--id-prefix'], {'default': ''}),
+         (SUPPRESS_HELP, ['--auto-id-prefix'], {'default': 'id'}),
          # Hidden options, for development use only:
          (SUPPRESS_HELP, ['--dump-settings'], {'action': 'store_true'}),
          (SUPPRESS_HELP, ['--dump-internals'], {'action': 'store_true'}),
@@ -445,14 +470,17 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
 
     settings_defaults = {'_disable_config': None,
                          '_source': None,
-                         '_destination': None}
+                         '_destination': None,
+                         '_config_files': None}
     """Defaults for settings that don't have command-line option equivalents."""
 
     relative_path_settings = ('warning_stream',)
 
     config_section = 'general'
 
-    version_template = '%%prog (Docutils %s)' % docutils.__version__
+    version_template = ('%%prog (Docutils %s [%s], Python %s, on %s)'
+                        % (docutils.__version__, docutils.__version_details__,
+                           sys.version.split()[0], sys.platform))
     """Default version message."""
 
     def __init__(self, components=(), defaults=None, read_config_files=None,
@@ -466,6 +494,9 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         self.lists = {}
         """Set of list-type settings."""
 
+        self.config_files = []
+        """List of paths of applied configuration files."""
+
         optparse.OptionParser.__init__(
             self, option_class=Option, add_help_option=None,
             formatter=optparse.TitledHelpFormatter(width=78),
@@ -476,13 +507,13 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         self.relative_path_settings = list(self.relative_path_settings)
         self.components = (self,) + tuple(components)
         self.populate_from_components(self.components)
-        self.set_defaults(**(defaults or {}))
+        self.set_defaults_from_dict(defaults or {})
         if read_config_files and not self.defaults['_disable_config']:
             try:
                 config_settings = self.get_standard_config_settings()
             except ValueError, error:
                 self.error(error)
-            self.set_defaults(**config_settings.__dict__)
+            self.set_defaults_from_dict(config_settings.__dict__)
 
     def populate_from_components(self, components):
         """
@@ -521,7 +552,19 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
             config_files = os.environ['DOCUTILSCONFIG'].split(os.pathsep)
         except KeyError:
             config_files = self.standard_config_files
-        return [os.path.expanduser(f) for f in config_files if f.strip()]
+
+        # If 'HOME' is not set, expandvars() requires the 'pwd' module which is
+        # not available under certain environments, for example, within
+        # mod_python.  The publisher ends up in here, and we need to publish
+        # from within mod_python.  Therefore we need to avoid expanding when we
+        # are in those environments.
+        expand = os.path.expanduser
+        if 'HOME' not in os.environ:
+            try:
+                import pwd
+            except ImportError:
+                expand = lambda x: x
+        return [expand(f) for f in config_files if f.strip()]
 
     def get_standard_config_settings(self):
         settings = Values()
@@ -533,6 +576,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         """Returns a dictionary containing appropriate config file settings."""
         parser = ConfigParser()
         parser.read(config_file, self)
+        self.config_files.extend(parser._files)
         base_path = os.path.dirname(config_file)
         applied = {}
         settings = Values()
@@ -554,6 +598,7 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         values._source, values._destination = self.check_args(args)
         make_paths_absolute(values.__dict__, self.relative_path_settings,
                             os.getcwd())
+        values._config_files = self.config_files
         return values
 
     def check_args(self, args):
@@ -573,9 +618,14 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                        'destination.  It will clobber the source file.')
         return source, destination
 
+    def set_defaults_from_dict(self, defaults):
+        self.defaults.update(defaults)
+
     def get_default_values(self):
         """Needed to get custom `Values` instances."""
-        return Values(self.defaults)
+        defaults = Values(self.defaults)
+        defaults._config_files = self.config_files
+        return defaults
 
     def get_option_by_dest(self, dest):
         """
@@ -610,11 +660,34 @@ configuration files.  See <http://docutils.sf.net/docs/user/config.html>,
 section "Old-Format Configuration Files".
 """
 
+    not_utf8_error = """\
+Unable to read configuration file "%s": content not encoded as UTF-8.
+Skipping "%s" configuration file.
+"""
+
+    def __init__(self, *args, **kwargs):
+        CP.ConfigParser.__init__(self, *args, **kwargs)
+
+        self._files = []
+        """List of paths of configuration files read."""
+
     def read(self, filenames, option_parser):
         if type(filenames) in (types.StringType, types.UnicodeType):
             filenames = [filenames]
         for filename in filenames:
-            CP.ConfigParser.read(self, filename)
+            try:
+                # Config files must be UTF-8-encoded:
+                fp = codecs.open(filename, 'r', 'utf-8')
+            except IOError:
+                continue
+            try:
+                CP.ConfigParser.readfp(self, fp, filename)
+            except UnicodeDecodeError:
+                sys.stderr.write(self.not_utf8_error % (filename, filename))
+                fp.close()
+                continue
+            fp.close()
+            self._files.append(filename)
             if self.has_section('options'):
                 self.handle_old_config(filename)
             self.validate_settings(filename, option_parser)
