@@ -1,7 +1,5 @@
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.69 $
-# Date: $Date: 2004/12/23 16:49:43 $
+# $Id: nodes.py 5033 2007-03-21 19:51:22Z wiemann $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -26,9 +24,9 @@ __docformat__ = 'reStructuredText'
 import sys
 import os
 import re
-import xml.dom.minidom
+import warnings
 from types import IntType, SliceType, StringType, UnicodeType, \
-     TupleType, ListType
+     TupleType, ListType, ClassType, TypeType
 from UserString import UserString
 
 
@@ -63,8 +61,17 @@ class Node:
         """
         return 1
 
-    def asdom(self, dom=xml.dom.minidom):
+    def __str__(self):
+        return self.__unicode__().encode('raw_unicode_escape')
+
+    def __unicode__(self):
+        # Override in subclass.
+        raise NotImplementedError
+
+    def asdom(self, dom=None):
         """Return a DOM **fragment** representation of this Node."""
+        if dom is None:
+            import xml.dom.minidom as dom
         domroot = dom.Document()
         return self._dom_node(domroot)
 
@@ -80,6 +87,10 @@ class Node:
         """Return a copy of self."""
         raise NotImplementedError
 
+    def deepcopy(self):
+        """Return a deep copy of self (also copying children)."""
+        raise NotImplementedError
+
     def setup_child(self, child):
         child.parent = self
         if self.document:
@@ -103,28 +114,37 @@ class Node:
         or replaced occurs after the current node, the old node will
         still be traversed, and any new nodes will not.
 
-        Within ``visit`` methods (and ``depart`` methods for 
+        Within ``visit`` methods (and ``depart`` methods for
         `walkabout()`), `TreePruningException` subclasses may be raised
         (`SkipChildren`, `SkipSiblings`, `SkipNode`, `SkipDeparture`).
 
         Parameter `visitor`: A `NodeVisitor` object, containing a
         ``visit`` implementation for each `Node` subclass encountered.
+
+        Return true if we should stop the traversal.
         """
+        stop = 0
         visitor.document.reporter.debug(
-            'calling dispatch_visit for %s' % self.__class__.__name__,
-            category='nodes.Node.walk')
+            'docutils.nodes.Node.walk calling dispatch_visit for %s'
+            % self.__class__.__name__)
         try:
-            visitor.dispatch_visit(self)
-        except (SkipChildren, SkipNode):
-            return
-        except SkipDeparture:           # not applicable; ignore
-            pass
-        children = self.get_children()
-        try:
-            for child in children[:]:
-                child.walk(visitor)
-        except SkipSiblings:
-            pass
+            try:
+                visitor.dispatch_visit(self)
+            except (SkipChildren, SkipNode):
+                return stop
+            except SkipDeparture:           # not applicable; ignore
+                pass
+            children = self.children
+            try:
+                for child in children[:]:
+                    if child.walk(visitor):
+                        stop = 1
+                        break
+            except SkipSiblings:
+                pass
+        except StopTraversal:
+            stop = 1
+        return stop
 
     def walkabout(self, visitor):
         """
@@ -135,32 +155,123 @@ class Node:
         Parameter `visitor`: A `NodeVisitor` object, containing a
         ``visit`` and ``depart`` implementation for each `Node`
         subclass encountered.
+
+        Return true if we should stop the traversal.
         """
         call_depart = 1
+        stop = 0
         visitor.document.reporter.debug(
-            'calling dispatch_visit for %s' % self.__class__.__name__,
-            category='nodes.Node.walkabout')
+            'docutils.nodes.Node.walkabout calling dispatch_visit for %s'
+            % self.__class__.__name__)
         try:
             try:
                 visitor.dispatch_visit(self)
             except SkipNode:
-                return
+                return stop
             except SkipDeparture:
                 call_depart = 0
-            children = self.get_children()
+            children = self.children
             try:
                 for child in children[:]:
-                    child.walkabout(visitor)
+                    if child.walkabout(visitor):
+                        stop = 1
+                        break
             except SkipSiblings:
                 pass
         except SkipChildren:
             pass
+        except StopTraversal:
+            stop = 1
         if call_depart:
             visitor.document.reporter.debug(
-                'calling dispatch_departure for %s' % self.__class__.__name__,
-                category='nodes.Node.walkabout')
+                'docutils.nodes.Node.walkabout calling dispatch_departure '
+                'for %s' % self.__class__.__name__)
             visitor.dispatch_departure(self)
+        return stop
 
+    def traverse(self, condition=None,
+                 include_self=1, descend=1, siblings=0, ascend=0):
+        """
+        Return an iterable containing
+
+        * self (if include_self is true)
+        * all descendants in tree traversal order (if descend is true)
+        * all siblings (if siblings is true) and their descendants (if
+          also descend is true)
+        * the siblings of the parent (if ascend is true) and their
+          descendants (if also descend is true), and so on
+
+        If `condition` is not None, the iterable contains only nodes
+        for which ``condition(node)`` is true.  If `condition` is a
+        node class ``cls``, it is equivalent to a function consisting
+        of ``return isinstance(node, cls)``.
+
+        If ascend is true, assume siblings to be true as well.
+
+        For example, given the following tree::
+
+            <paragraph>
+                <emphasis>      <--- emphasis.traverse() and
+                    <strong>    <--- strong.traverse() are called.
+                        Foo
+                    Bar
+                <reference name="Baz" refid="baz">
+                    Baz
+
+        Then list(emphasis.traverse()) equals ::
+
+            [<emphasis>, <strong>, <#text: Foo>, <#text: Bar>]
+
+        and list(strong.traverse(ascend=1)) equals ::
+
+            [<strong>, <#text: Foo>, <#text: Bar>, <reference>, <#text: Baz>]
+        """
+        r = []
+        if ascend:
+            siblings=1
+        # Check if `condition` is a class (check for TypeType for Python
+        # implementations that use only new-style classes, like PyPy).
+        if isinstance(condition, (ClassType, TypeType)):
+            node_class = condition
+            def condition(node, node_class=node_class):
+                return isinstance(node, node_class)
+        if include_self and (condition is None or condition(self)):
+            r.append(self)
+        if descend and len(self.children):
+            for child in self:
+                r.extend(child.traverse(
+                    include_self=1, descend=1, siblings=0, ascend=0,
+                    condition=condition))
+        if siblings or ascend:
+            node = self
+            while node.parent:
+                index = node.parent.index(node)
+                for sibling in node.parent[index+1:]:
+                    r.extend(sibling.traverse(include_self=1, descend=descend,
+                                              siblings=0, ascend=0,
+                                              condition=condition))
+                if not ascend:
+                    break
+                else:
+                    node = node.parent
+        return r
+
+    def next_node(self, condition=None,
+                  include_self=0, descend=1, siblings=0, ascend=0):
+        """
+        Return the first node in the iterable returned by traverse(),
+        or None if the iterable is empty.
+
+        Parameter list is the same as of traverse.  Note that
+        include_self defaults to 0, though.
+        """
+        iterable = self.traverse(condition=condition,
+                                 include_self=include_self, descend=descend,
+                                 siblings=siblings, ascend=ascend)
+        try:
+            return iterable[0]
+        except IndexError:
+            return None
 
 class Text(Node, UserString):
 
@@ -172,6 +283,9 @@ class Text(Node, UserString):
 
     tagname = '#text'
 
+    children = ()
+    """Text nodes have no children, and cannot have children."""
+
     def __init__(self, data, rawsource=''):
         UserString.__init__(self, data)
 
@@ -199,9 +313,15 @@ class Text(Node, UserString):
     def astext(self):
         return self.data
 
+    def __unicode__(self):
+        return self.data
+
     def copy(self):
         return self.__class__(self.data)
 
+    def deepcopy(self):
+        return self.copy()
+
     def pformat(self, indent='    ', level=0):
         result = []
         indent = indent * level
@@ -209,10 +329,6 @@ class Text(Node, UserString):
             result.append(indent + line + '\n')
         return ''.join(result)
 
-    def get_children(self):
-        """Text nodes have no children. Return []."""
-        return []
-
 
 class Element(Node):
 
@@ -225,6 +341,12 @@ class Element(Node):
 
         element['att'] = 'value'
 
+    There are two special attributes: 'ids' and 'names'.  Both are
+    lists of unique identifiers, and names serve as human interfaces
+    to IDs.  Names are case- and whitespace-normalized (see the
+    fully_normalize_name() function), and IDs conform to the regular
+    expression ``[a-z](-?[a-z0-9]+)*`` (see the make_id() function).
+
     Elements also emulate lists for child nodes (element nodes and/or text
     nodes), indexing by integer.  To get the first child node, use::
 
@@ -245,6 +367,10 @@ class Element(Node):
     This is equivalent to ``element.extend([node1, node2])``.
     """
 
+    list_attributes = ('ids', 'classes', 'names', 'dupnames', 'backrefs')
+    """List attributes, automatically initialized to empty lists for
+    all nodes."""
+
     tagname = None
     """The element generic identifier. If None, it is set as an instance
     attribute to the name of the class."""
@@ -264,17 +390,26 @@ class Element(Node):
         self.attributes = {}
         """Dictionary of attribute {name: value}."""
 
+        # Initialize list attributes.
+        for att in self.list_attributes:
+            self.attributes[att] = []
+
         for att, value in attributes.items():
-            self.attributes[att.lower()] = value
+            att = att.lower()
+            if att in self.list_attributes:
+                # mutable list; make a copy for this node
+                self.attributes[att] = value[:]
+            else:
+                self.attributes[att] = value
 
         if self.tagname is None:
             self.tagname = self.__class__.__name__
 
     def _dom_node(self, domroot):
         element = domroot.createElement(self.tagname)
-        for attribute, value in self.attributes.items():
+        for attribute, value in self.attlist():
             if isinstance(value, ListType):
-                value = ' '.join(['%s' % v for v in value])
+                value = ' '.join([serial_escape('%s' % v) for v in value])
             element.setAttribute(attribute, '%s' % value)
         for child in self.children:
             element.appendChild(child._dom_node(domroot))
@@ -287,27 +422,24 @@ class Element(Node):
             if len(data) > 60:
                 data = data[:56] + ' ...'
                 break
-        if self.hasattr('name'):
+        if self['names']:
             return '<%s "%s": %s>' % (self.__class__.__name__,
-                                      self.attributes['name'], data)
+                                      '; '.join(self['names']), data)
         else:
             return '<%s: %s>' % (self.__class__.__name__, data)
 
     def shortrepr(self):
-        if self.hasattr('name'):
+        if self['names']:
             return '<%s "%s"...>' % (self.__class__.__name__,
-                                      self.attributes['name'])
+                                     '; '.join(self['names']))
         else:
             return '<%s...>' % self.tagname
 
-    def __str__(self):
-        return self.__unicode__().encode('raw_unicode_escape')
-
     def __unicode__(self):
         if self.children:
             return u'%s%s%s' % (self.starttag(),
-                                 ''.join([str(c) for c in self.children]),
-                                 self.endtag())
+                                ''.join([unicode(c) for c in self.children]),
+                                self.endtag())
         else:
             return self.emptytag()
 
@@ -317,7 +449,7 @@ class Element(Node):
             if value is None:           # boolean attribute
                 parts.append(name)
             elif isinstance(value, ListType):
-                values = ['%s' % v for v in value]
+                values = [serial_escape('%s' % v) for v in value]
                 parts.append('%s="%s"' % (name, ' '.join(values)))
             else:
                 parts.append('%s="%s"' % (name, value))
@@ -382,20 +514,24 @@ class Element(Node):
     def __iadd__(self, other):
         """Append a node or a list of nodes to `self.children`."""
         if isinstance(other, Node):
-            self.setup_child(other)
-            self.children.append(other)
+            self.append(other)
         elif other is not None:
-            for node in other:
-                self.setup_child(node)
-            self.children.extend(other)
+            self.extend(other)
         return self
 
     def astext(self):
         return self.child_text_separator.join(
               [child.astext() for child in self.children])
 
+    def non_default_attributes(self):
+        atts = {}
+        for key, value in self.attributes.items():
+            if self.is_not_default(key):
+                atts[key] = value
+        return atts
+
     def attlist(self):
-        attlist = self.attributes.items()
+        attlist = self.non_default_attributes().items()
         attlist.sort()
         return attlist
 
@@ -420,8 +556,7 @@ class Element(Node):
 
     def extend(self, item):
         for node in item:
-            self.setup_child(node)
-        self.children.extend(item)
+            self.append(node)
 
     def insert(self, index, item):
         if isinstance(item, Node):
@@ -439,6 +574,27 @@ class Element(Node):
     def index(self, item):
         return self.children.index(item)
 
+    def is_not_default(self, key):
+        if self[key] == [] and key in self.list_attributes:
+            return 0
+        else:
+            return 1
+
+    def update_basic_atts(self, dict):
+        """
+        Update basic attributes ('ids', 'names', 'classes',
+        'dupnames', but not 'source') from node or dictionary `dict`.
+        """
+        if isinstance(dict, Node):
+            dict = dict.attributes
+        for att in ('ids', 'classes', 'names', 'dupnames'):
+            for value in dict.get(att, []):
+                if not value in self[att]:
+                    self[att].append(value)
+
+    def clear(self):
+        self.children = []
+
     def replace(self, old, new):
         """Replace one child `Node` with another child or children."""
         index = self.index(old)
@@ -448,6 +604,28 @@ class Element(Node):
         elif new is not None:
             self[index:index+1] = new
 
+    def replace_self(self, new):
+        """
+        Replace `self` node with `new`, where `new` is a node or a
+        list of nodes.
+        """
+        update = new
+        if not isinstance(new, Node):
+            # `new` is a list; update first child.
+            try:
+                update = new[0]
+            except IndexError:
+                update = None
+        if isinstance(update, Element):
+            update.update_basic_atts(self)
+        else:
+            # `update` is a Text node or `new` is an empty list.
+            # Assert that we aren't losing any attributes.
+            for att in ('ids', 'names', 'classes', 'dupnames'):
+                assert not self[att], \
+                       'Losing "%s" attribute: %s' % (att, self[att])
+        self.parent.replace(self, new)
+
     def first_child_matching_class(self, childclass, start=0, end=sys.maxint):
         """
         Return the index of the first child whose class exactly matches.
@@ -482,12 +660,10 @@ class Element(Node):
         if not isinstance(childclass, TupleType):
             childclass = (childclass,)
         for index in range(start, min(len(self), end)):
-            match = 0
             for c in childclass:
                 if isinstance(self.children[index], c):
-                    match = 1
                     break
-            if not match:
+            else:
                 return index
         return None
 
@@ -496,17 +672,38 @@ class Element(Node):
                        [child.pformat(indent, level+1)
                         for child in self.children])
 
-    def get_children(self):
-        """Return this element's children."""
-        return self.children
-
     def copy(self):
         return self.__class__(**self.attributes)
 
+    def deepcopy(self):
+        copy = self.copy()
+        copy.extend([child.deepcopy() for child in self.children])
+        return copy
+
     def set_class(self, name):
-        """Add a new name to the "class" attribute."""
-        self.attributes['class'] = (self.attributes.get('class', '') + ' '
-                                    + name.lower()).strip()
+        """Add a new class to the "classes" attribute."""
+        warnings.warn('docutils.nodes.Element.set_class deprecated; '
+                      "append to Element['classes'] list attribute directly",
+                      DeprecationWarning, stacklevel=2)
+        assert ' ' not in name
+        self['classes'].append(name.lower())
+
+    def note_referenced_by(self, name=None, id=None):
+        """Note that this Element has been referenced by its name
+        `name` or id `id`."""
+        self.referenced = 1
+        # Element.expect_referenced_by_* dictionaries map names or ids
+        # to nodes whose ``referenced`` attribute is set to true as
+        # soon as this node is referenced by the given name or id.
+        # Needed for target propagation.
+        by_name = getattr(self, 'expect_referenced_by_name', {}).get(name)
+        by_id = getattr(self, 'expect_referenced_by_id', {}).get(id)
+        if by_name:
+            assert name is not None
+            by_name.referenced = 1
+        if by_id:
+            assert id is not None
+            by_id.referenced = 1
 
 
 class TextElement(Element):
@@ -514,7 +711,7 @@ class TextElement(Element):
     """
     An element which directly contains text.
 
-    Its children are all `Text` or `TextElement` subclass nodes.  You can
+    Its children are all `Text` or `Inline` subclass nodes.  You can
     check whether an element's context is inline simply by checking whether
     its immediate parent is a `TextElement` instance (including subclasses).
     This is handy for nodes like `image` that can appear both inline and as
@@ -557,7 +754,7 @@ class Resolvable:
 class BackLinkable:
 
     def add_backref(self, refid):
-        self.setdefault('backrefs', []).append(refid)
+        self['backrefs'].append(refid)
 
 
 # ====================
@@ -568,15 +765,12 @@ class Root: pass
 
 class Titular: pass
 
-class PreDecorative:
-    """Category of Node which may occur before Decorative Nodes."""
-
-class PreBibliographic(PreDecorative):
+class PreBibliographic:
     """Category of Node which may occur before Bibliographic Nodes."""
 
-class Bibliographic(PreDecorative): pass
+class Bibliographic: pass
 
-class Decorative: pass
+class Decorative(PreBibliographic): pass
 
 class Structural: pass
 
@@ -584,7 +778,8 @@ class Body: pass
 
 class General(Body): pass
 
-class Sequential(Body): pass
+class Sequential(Body):
+    """List-like elements."""
 
 class Admonition(Body): pass
 
@@ -600,12 +795,15 @@ class Inline: pass
 
 class Referential(Resolvable): pass
 
+
 class Targetable(Resolvable):
 
     referenced = 0
 
     indirect_reference_name = None
-    """Holds the whitespace_normalized_name (contains mixed case) of a target"""
+    """Holds the whitespace_normalized_name (contains mixed case) of a target.
+    Required for MoinMoin/reST compatibility."""
+
 
 class Labeled:
     """Contains a `label` as its first element."""
@@ -617,6 +815,13 @@ class Labeled:
 
 class document(Root, Structural, Element):
 
+    """
+    The document root element.
+
+    Do not instantiate this class directly; use
+    `docutils.utils.new_document()` instead.
+    """
+
     def __init__(self, settings, reporter, *args, **kwargs):
         Element.__init__(self, *args, **kwargs)
 
@@ -632,12 +837,6 @@ class document(Root, Structural, Element):
         self.reporter = reporter
         """System message generator."""
 
-        self.external_targets = []
-        """List of external target nodes."""
-
-        self.internal_targets = []
-        """List of internal target nodes."""
-
         self.indirect_targets = []
         """List of indirect target nodes."""
 
@@ -664,22 +863,12 @@ class document(Root, Structural, Element):
         self.ids = {}
         """Mapping of ids to nodes."""
 
-        self.substitution_refs = {}
-        """Mapping of substitution names to lists of substitution_reference
-        nodes."""
-
         self.footnote_refs = {}
         """Mapping of footnote labels to lists of footnote_reference nodes."""
 
         self.citation_refs = {}
         """Mapping of citation labels to lists of citation_reference nodes."""
 
-        self.anonymous_targets = []
-        """List of anonymous target nodes."""
-
-        self.anonymous_refs = []
-        """List of anonymous reference nodes."""
-
         self.autofootnotes = []
         """List of auto-numbered footnote nodes."""
 
@@ -717,30 +906,46 @@ class document(Root, Structural, Element):
         self.transformer = docutils.transforms.Transformer(self)
         """Storage for transforms to be applied to this document."""
 
+        self.decoration = None
+        """Document's `decoration` node."""
+
         self.document = self
 
-    def asdom(self, dom=xml.dom.minidom):
+    def __getstate__(self):
+        """
+        Return dict with unpicklable references removed.
+        """
+        state = self.__dict__.copy()
+        state['reporter'] = None
+        state['transformer'] = None
+        return state
+
+    def asdom(self, dom=None):
         """Return a DOM representation of this document."""
+        if dom is None:
+            import xml.dom.minidom as dom
         domroot = dom.Document()
         domroot.appendChild(self._dom_node(domroot))
         return domroot
 
     def set_id(self, node, msgnode=None):
-        if node.has_key('id'):
-            id = node['id']
+        for id in node['ids']:
             if self.ids.has_key(id) and self.ids[id] is not node:
                 msg = self.reporter.severe('Duplicate ID: "%s".' % id)
                 if msgnode != None:
                     msgnode += msg
-        else:
-            if node.has_key('name'):
-                id = make_id(node['name'])
+        if not node['ids']:
+            for name in node['names']:
+                id = self.settings.id_prefix + make_id(name)
+                if id and not self.ids.has_key(id):
+                    break
             else:
                 id = ''
-            while not id or self.ids.has_key(id):
-                id = 'id%s' % self.id_start
-                self.id_start += 1
-            node['id'] = id
+                while not id or self.ids.has_key(id):
+                    id = (self.settings.id_prefix +
+                          self.settings.auto_id_prefix + str(self.id_start))
+                    self.id_start += 1
+            node['ids'].append(id)
         self.ids[id] = node
         return id
 
@@ -752,16 +957,17 @@ class document(Root, Structural, Element):
 
         The following state transition table shows how `self.nameids` ("ids")
         and `self.nametypes` ("types") change with new input (a call to this
-        method), and what actions are performed:
+        method), and what actions are performed ("implicit"-type system
+        messages are INFO/1, and "explicit"-type system messages are ERROR/3):
 
         ====  =====  ========  ========  =======  ====  =====  =====
          Old State    Input          Action        New State   Notes
         -----------  --------  -----------------  -----------  -----
         ids   types  new type  sys.msg.  dupname  ids   types
         ====  =====  ========  ========  =======  ====  =====  =====
-        --    --     explicit  --        --       new   True
-        --    --     implicit  --        --       new   False
-        None  False  explicit  --        --       new   True
+        -     -      explicit  -         -        new   True
+        -     -      implicit  -         -        new   False
+        None  False  explicit  -         -        new   True
         old   False  explicit  implicit  old      new   True
         None  True   explicit  explicit  new      None  True
         old   True   explicit  explicit  new,old  None  True   [#]_
@@ -775,8 +981,7 @@ class document(Root, Structural, Element):
            both old and new targets are external and refer to identical URIs.
            The new target is invalidated regardless.
         """
-        if node.has_key('name'):
-            name = node['name']
+        for name in node['names']:
             if self.nameids.has_key(name):
                 self.set_duplicate_name_id(node, id, name, msgnode, explicit)
             else:
@@ -794,30 +999,30 @@ class document(Root, Structural, Element):
                     old_node = self.ids[old_id]
                     if node.has_key('refuri'):
                         refuri = node['refuri']
-                        if old_node.has_key('name') \
+                        if old_node['names'] \
                                and old_node.has_key('refuri') \
                                and old_node['refuri'] == refuri:
                             level = 1   # just inform if refuri's identical
                     if level > 1:
-                        dupname(old_node)
+                        dupname(old_node, name)
                         self.nameids[name] = None
                 msg = self.reporter.system_message(
                     level, 'Duplicate explicit target name: "%s".' % name,
                     backrefs=[id], base_node=node)
                 if msgnode != None:
                     msgnode += msg
-                dupname(node)
+                dupname(node, name)
             else:
                 self.nameids[name] = id
                 if old_id is not None:
                     old_node = self.ids[old_id]
-                    dupname(old_node)
+                    dupname(old_node, name)
         else:
             if old_id is not None and not old_explicit:
                 self.nameids[name] = None
                 old_node = self.ids[old_id]
-                dupname(old_node)
-            dupname(node)
+                dupname(old_node, name)
+            dupname(node, name)
         if not explicit or (not old_explicit and old_id is not None):
             msg = self.reporter.info(
                 'Duplicate implicit target name: "%s".' % name,
@@ -843,23 +1048,13 @@ class document(Root, Structural, Element):
     def note_refid(self, node):
         self.refids.setdefault(node['refid'], []).append(node)
 
-    def note_external_target(self, target):
-        self.external_targets.append(target)
-
-    def note_internal_target(self, target):
-        self.internal_targets.append(target)
-
     def note_indirect_target(self, target):
         self.indirect_targets.append(target)
-        if target.has_key('name'):
+        if target['names']:
             self.note_refname(target)
 
     def note_anonymous_target(self, target):
         self.set_id(target)
-        self.anonymous_targets.append(target)
-
-    def note_anonymous_ref(self, ref):
-        self.anonymous_refs.append(ref)
 
     def note_autofootnote(self, footnote):
         self.set_id(footnote)
@@ -895,7 +1090,7 @@ class document(Root, Structural, Element):
         self.note_refname(ref)
 
     def note_substitution_def(self, subdef, def_name, msgnode=None):
-        name = subdef['name'] = whitespace_normalize_name(def_name)
+        name = whitespace_normalize_name(def_name)
         if self.substitution_defs.has_key(name):
             msg = self.reporter.error(
                   'Duplicate substitution definition name: "%s".' % name,
@@ -903,15 +1098,14 @@ class document(Root, Structural, Element):
             if msgnode != None:
                 msgnode += msg
             oldnode = self.substitution_defs[name]
-            dupname(oldnode)
+            dupname(oldnode, name)
         # keep only the last definition:
         self.substitution_defs[name] = subdef
         # case-insensitive mapping:
         self.substitution_names[fully_normalize_name(name)] = name
 
     def note_substitution_ref(self, subref, refname):
-        name = subref['refname'] = whitespace_normalize_name(refname)
-        self.substitution_refs.setdefault(name, []).append(subref)
+        subref['refname'] = whitespace_normalize_name(refname)
 
     def note_pending(self, pending, priority=None):
         self.transformer.add_pending(pending, priority)
@@ -933,6 +1127,16 @@ class document(Root, Structural, Element):
         return self.__class__(self.settings, self.reporter,
                               **self.attributes)
 
+    def get_decoration(self):
+        if not self.decoration:
+            self.decoration = decoration()
+            index = self.first_child_not_matching_class(Titular)
+            if index is None:
+                self.append(self.decoration)
+            else:
+                self.insert(index, self.decoration)
+        return self.decoration
+
 
 # ================
 #  Title Elements
@@ -964,7 +1168,19 @@ class copyright(Bibliographic, TextElement): pass
 #  Decorative Elements
 # =====================
 
-class decoration(Decorative, Element): pass
+class decoration(Decorative, Element):
+
+    def get_header(self):
+        if not len(self.children) or not isinstance(self.children[0], header):
+            self.insert(0, header())
+        return self.children[0]
+
+    def get_footer(self):
+        if not len(self.children) or not isinstance(self.children[-1], footer):
+            self.append(footer())
+        return self.children[-1]
+
+
 class header(Decorative, Element): pass
 class footer(Decorative, Element): pass
 
@@ -1016,6 +1232,7 @@ class transition(Structural, Element): pass
 
 class paragraph(General, TextElement): pass
 class compound(General, Element): pass
+class container(General, Element): pass
 class bullet_list(Sequential, Element): pass
 class enumerated_list(Sequential, Element): pass
 class list_item(Part, Element): pass
@@ -1061,7 +1278,7 @@ class doctest_block(General, FixedTextElement): pass
 class line_block(General, Element): pass
 
 
-class line(General, TextElement):
+class line(Part, TextElement):
 
     indent = None
 
@@ -1081,8 +1298,8 @@ class admonition(Admonition, Element): pass
 class comment(Special, Invisible, FixedTextElement): pass
 class substitution_definition(Special, Invisible, TextElement): pass
 class target(Special, Invisible, Inline, TextElement, Targetable): pass
-class footnote(General, Element, Labeled, BackLinkable): pass
-class citation(General, Element, Labeled, BackLinkable): pass
+class footnote(General, BackLinkable, Element, Labeled, Targetable): pass
+class citation(General, BackLinkable, Element, Labeled, Targetable): pass
 class label(Part, TextElement): pass
 class figure(General, Element): pass
 class caption(Part, TextElement): pass
@@ -1096,7 +1313,14 @@ class row(Part, Element): pass
 class entry(Part, Element): pass
 
 
-class system_message(Special, PreBibliographic, Element, BackLinkable):
+class system_message(Special, BackLinkable, PreBibliographic, Element):
+
+    """
+    System message element.
+
+    Do not instantiate this class directly; use
+    ``document.reporter.info/warning/error/severe()`` instead.
+    """
 
     def __init__(self, message=None, *children, **attributes):
         if message:
@@ -1210,7 +1434,7 @@ class superscript(Inline, TextElement): pass
 class subscript(Inline, TextElement): pass
 
 
-class image(General, Inline, TextElement):
+class image(General, Inline, Element):
 
     def astext(self):
         return self.get('alt', '')
@@ -1231,7 +1455,7 @@ node_class_names = """
         authors
     block_quote bullet_list
     caption caution citation citation_reference classifier colspec comment
-        compound contact copyright
+        compound contact container copyright
     danger date decoration definition definition_list definition_list_item
         description docinfo doctest_block document
     emphasis entry enumerated_list error
@@ -1306,8 +1530,8 @@ class NodeVisitor:
         node_name = node.__class__.__name__
         method = getattr(self, 'visit_' + node_name, self.unknown_visit)
         self.document.reporter.debug(
-            'calling %s for %s' % (method.__name__, node_name),
-            category='nodes.NodeVisitor.dispatch_visit')
+            'docutils.nodes.NodeVisitor.dispatch_visit calling %s for %s'
+            % (method.__name__, node_name))
         return method(node)
 
     def dispatch_departure(self, node):
@@ -1319,8 +1543,8 @@ class NodeVisitor:
         node_name = node.__class__.__name__
         method = getattr(self, 'depart_' + node_name, self.unknown_departure)
         self.document.reporter.debug(
-            'calling %s for %s' % (method.__name__, node_name),
-            category='nodes.NodeVisitor.dispatch_departure')
+            'docutils.nodes.NodeVisitor.dispatch_departure calling %s for %s'
+            % (method.__name__, node_name))
         return method(node)
 
     def unknown_visit(self, node):
@@ -1357,6 +1581,7 @@ class SparseNodeVisitor(NodeVisitor):
     subclasses), subclass `NodeVisitor` instead.
     """
 
+
 class GenericNodeVisitor(NodeVisitor):
 
     """
@@ -1398,10 +1623,11 @@ def _add_node_class_names(names):
         setattr(GenericNodeVisitor, "visit_" + _name, _call_default_visit)
         setattr(GenericNodeVisitor, "depart_" + _name, _call_default_departure)
         setattr(SparseNodeVisitor, 'visit_' + _name, _nop)
-        setattr(SparseNodeVisitor, 'depart' + _name, _nop)
+        setattr(SparseNodeVisitor, 'depart_' + _name, _nop)
 
 _add_node_class_names(node_class_names)
 
+
 class TreeCopyVisitor(GenericNodeVisitor):
 
     """
@@ -1492,6 +1718,19 @@ class NodeFound(TreePruningException):
     pass
 
 
+class StopTraversal(TreePruningException):
+
+    """
+    Stop the traversal alltogether.  The current node's ``depart_...`` method
+    is not affected.  The parent nodes ``depart_...`` methods are also called
+    as usual.  No other nodes are visited.  This is an alternative to
+    NodeFound that does not cause exception handling to trickle up to the
+    caller.
+    """
+
+    pass
+
+
 def make_id(string):
     """
     Convert `string` into an identifier and return it.
@@ -1534,9 +1773,12 @@ def make_id(string):
 _non_id_chars = re.compile('[^a-z0-9]+')
 _non_id_at_ends = re.compile('^[-0-9]+|-+$')
 
-def dupname(node):
-    node['dupname'] = node['name']
-    del node['name']
+def dupname(node, name):
+    node['dupnames'].append(name)
+    node['names'].remove(name)
+    # Assume that this method is referenced, even though it isn't; we
+    # don't want to throw unnecessary system_messages.
+    node.referenced = 1
 
 def fully_normalize_name(name):
     """Return a case- and whitespace-normalized name."""
@@ -1545,3 +1787,15 @@ def fully_normalize_name(name):
 def whitespace_normalize_name(name):
     """Return a whitespace-normalized name."""
     return ' '.join(name.split())
+
+def serial_escape(value):
+    """Escape string values that are elements of a list, for serialization."""
+    return value.replace('\\', r'\\').replace(' ', r'\ ')
+
+# 
+#
+# Local Variables:
+# indent-tabs-mode: nil
+# sentence-end-double-space: t
+# fill-column: 78
+# End:
